unit Cpu2;

interface

uses
  Classes, SysUtils;

type
  TBitWidth = (bw8, bw16, bw32, bw64, bw128, bw256, bw512, bw1024);

  TInstruction = (inNop, inHalt, inLoad8, inLoad16, inStore8, inStore16,
    inAdd8, inAdd16, inJump, inJumpNZ);

  { TValue }

  TValue = record
    BitWidth: TBitWidth;
    class function Create8(Value: Byte): TValue; static;
    class function Create16(Value: Word): TValue; static;
    class function Create32(Value: DWord): TValue; static;
    class function Create64(Value: QWord): TValue; static;
    case TBitWidth of
      bw8: (Value8: Byte);
      bw16: (Value16: Word);
      bw32: (Value32: DWord);
      bw64: (Value64: QWord);
  end;

  { TCpu2 }

  TCpu2<TAddress> = class
  public
  type
    TReadMemEvent = function (Address: TAddress): TValue of object;
  private
    FReadMem: TReadMemEvent;
    function ReadMem8(Address: TAddress): Byte;
    function ReadMem16(Address: TAddress): Word;
    function ReadMemAddress(Address: TAddress): TAddress;
    function ReadMemPc8: Byte;
    function ReadMemPc16: Word;
    function ReadMemPcAddress: TAddress;
    procedure WriteMem8(Address: TAddress; Data: Byte);
    procedure WriteMem16(Address: TAddress; Data: Word);
    procedure WriteMemAddress(Address: TAddress; Data: TAddress);
  public
    Halted: Boolean;
    A: TValue;
    PC: TAddress;
    Zero: Boolean;
    procedure Run;
    procedure Step;
    property ReadMem: TReadMemEvent read FReadMem write FReadMem;
  end;


implementation

{ TCpu2 }

function TCpu2<TAddress>.ReadMem8(Address: TAddress): Byte;
begin

end;

function TCpu2<TAddress>.ReadMem16(Address: TAddress): Word;
begin

end;

function TCpu2<TAddress>.ReadMemAddress(Address: TAddress): TAddress;
begin

end;

function TCpu2<TAddress>.ReadMemPc8: Byte;
begin
  Result := ReadMem8(PC);
  Inc(PC, SizeOf(Byte));
end;

function TCpu2<TAddress>.ReadMemPc16: Word;
begin
  Result := ReadMem16(PC);
  Inc(PC, SizeOf(Word));
end;

function TCpu2<TAddress>.ReadMemPcAddress: TAddress;
begin
  Result := ReadMemAddress(PC);
  Inc(PC, SizeOf(TAddress));
end;

procedure TCpu2<TAddress>.WriteMem8(Address: TAddress; Data: Byte);
begin

end;

procedure TCpu2<TAddress>.WriteMem16(Address: TAddress; Data: Word);
begin

end;

procedure TCpu2<TAddress>.WriteMemAddress(Address: TAddress; Data: TAddress);
begin

end;

procedure TCpu2<TAddress>.Run;
begin
  while not Halted do
    Step;
end;

procedure TCpu2<TAddress>.Step;
var
  Opcode: Byte;
  Instruction: TInstruction;
  Addr: TAddress;
begin
  Opcode := ReadMemPc8;
  Instruction := TInstruction(Opcode);
  case Instruction of
    inNop: ;
    inHalt: Halted := True;
    inLoad8: A.Value8 := ReadMem8(ReadMemPcAddress);
    inLoad16: A.Value16 := ReadMem16(ReadMemPcAddress);
    inStore8: WriteMem8(ReadMemPcAddress, A.Value8);
    inStore16: WriteMem16(ReadMemPcAddress, A.Value16);
    inAdd8: A.Value8 := A.Value8 + ReadMem8(ReadMemPcAddress);
    inAdd16: A.Value16 := A.Value16 + ReadMem16(ReadMemPcAddress);
    inJump: PC := ReadMemAddress;
    inJumpNZ: begin
      Addr := ReadMemAddress;
      if Zero then PC := Addr;
    end;
  end;
end;

{ TValue }

class function TValue.Create8(Value: Byte): TValue;
begin
  Result.Value8 := Value;
end;

class function TValue.Create16(Value: Word): TValue;
begin
  Result.Value16 := Value;
end;

class function TValue.Create32(Value: DWord): TValue;
begin
  Result.Value32 := Value;
end;

class function TValue.Create64(Value: QWord): TValue;
begin
  Result.Value64 := Value;
end;

end.

LD8  A, n
LD16 B, nn
LD32 B, nnnn
LD   A.b, n
LD   B.w, nn
LD   B.d, nnnn
LD   A8, n
LD   B16, nn
LD   B32, nn
LD   B32, A32
LD32 B, A
LD   B, A


IN   A, (n)
OUT  (n), A
LD   A, (n)
LD   (n), A
