unit CpuBitWidth;

interface

uses
  Classes, SysUtils, AddressableArea;

type
  TInstruction = (inNop, inHalt,
    inLoad1, inLoad2, inLoad4, inLoad8,
    inLoadMem1, inLoadMem2, inLoadMem4, inLoadMem8,
    inStoreMem1, inStoreMem2, inStoreMem4, inStoreMem8,
    inInput1, inInput2, inInput4, inInput8,
    inOutput1, inOutput2, inOutput4, inOutput8);

  { TCpuBitWidth1 }

  TCpuBitWidth1 = class
    Memory: TAddressableArea1;
    IO: TAddressableArea1;
    Halted: Boolean;
    PC: Byte;
    A: Byte;
    function Read1: Byte;
    procedure Write1(Data: Byte);
    procedure Reset;
    procedure Run;
    procedure Step;
  end;

  { TCpuBitWidth2 }

  TCpuBitWidth2 = class
    Memory: TAddressableArea2;
    IO: TAddressableArea2;
    Halted: Boolean;
    PC: Word;
    A: Word;
    function Read1: Byte;
    function Read2: Word;
    procedure Write1(Data: Byte);
    procedure Write2(Data: Word);
    procedure Reset;
    procedure Run;
    procedure Step;
  end;

  { TCpuBitWidth4 }

  TCpuBitWidth4 = class
    Memory: TAddressableArea4;
    IO: TAddressableArea4;
    Halted: Boolean;
    PC: Cardinal;
    A: Cardinal;
    function Read1: Byte;
    function Read2: Word;
    function Read4: Cardinal;
    procedure Reset;
    procedure Run;
    procedure Step;
  end;

  { TCpuBitWidth8 }

  TCpuBitWidth8 = class
    Memory: TAddressableArea8;
    IO: TAddressableArea8;
    Halted: Boolean;
    PC: DWord;
    A: DWord;
    function Read1: Byte;
    function Read2: Word;
    function Read4: Cardinal;
    function Read8: DWord;
    procedure Reset;
    procedure Run;
    procedure Step;
  end;

implementation

{ TCpuBitWidth4 }

function TCpuBitWidth4.Read1: Byte;
begin
  Result := Memory.Read1(PC);
  Inc(PC);
end;

function TCpuBitWidth4.Read2: Word;
begin
  Result := Memory.Read2(PC);
  Inc(PC, 2);
end;

function TCpuBitWidth4.Read4: Cardinal;
begin
  Result := Memory.Read4(PC);
  Inc(PC, 4);
end;

procedure TCpuBitWidth4.Reset;
begin
  PC := 0;
  A := 0;
end;

procedure TCpuBitWidth4.Run;
begin
  Reset;
  while not Halted do
    Step;
end;

procedure TCpuBitWidth4.Step;
var
  Opcode: TInstruction;
begin
  Opcode := TInstruction(Read1);
  case Opcode of
    inNop: ;
    inHalt: Halted := True;
    inLoad1: A := Read1;
    inLoad2: A := Read2;
    inLoad4: A := Read4;
    inLoadMem1: A := Memory.Read1(Read4);
    inLoadMem2: A := Memory.Read2(Read4);
    inLoadMem4: A := Memory.Read4(Read4);
    inStoreMem1: Memory.Write1(Read4, A);
    inStoreMem2: Memory.Write2(Read4, A);
    inStoreMem4: Memory.Write4(Read4, A);
    inInput1: A := IO.Read1(Read4);
    inInput2: A := IO.Read2(Read4);
    inInput4: A := IO.Read4(Read4);
    inOutput1: IO.Write1(Read4, A);
    inOutput2: IO.Write2(Read4, A);
    inOutput4: IO.Write4(Read4, A);
  end;
end;

{ TCpuBitWidth2 }

function TCpuBitWidth2.Read1: Byte;
begin
  Result := Memory.Read1(PC);
  Inc(PC);
end;

function TCpuBitWidth2.Read2: Word;
begin
  Result := Memory.Read2(PC);
  Inc(PC, 2);
end;

procedure TCpuBitWidth2.Write1(Data: Byte);
begin
  Memory.Write1(PC, Data);
  Inc(PC);
end;

procedure TCpuBitWidth2.Write2(Data: Word);
begin
  Memory.Write2(PC, Data);
  Inc(PC, 2);
end;

procedure TCpuBitWidth2.Reset;
begin
  PC := 0;
  A := 0;
end;

procedure TCpuBitWidth2.Run;
begin
  Reset;
  while not Halted do
    Step;
end;

procedure TCpuBitWidth2.Step;
var
  Opcode: TInstruction;
begin
  Opcode := TInstruction(Read1);
  case Opcode of
    inNop: ;
    inHalt: Halted := True;
    inLoad1: A := Read1;
    inLoad2: A := Read2;
    inLoadMem1: A := Memory.Read1(Read2);
    inLoadMem2: A := Memory.Read2(Read2);
    inStoreMem1: Memory.Write1(Read2, A);
    inStoreMem2: Memory.Write2(Read2, A);
    inInput1: A := IO.Read1(Read2);
    inInput2: A := IO.Read2(Read2);
    inOutput1: IO.Write1(Read2, A);
    inOutput2: IO.Write2(Read2, A);
  end;
end;

{ TCpuBitWidth8 }

function TCpuBitWidth8.Read1: Byte;
begin
  Result := Memory.Read1(PC);
  Inc(PC);
end;

function TCpuBitWidth8.Read2: Word;
begin
  Result := Memory.Read2(PC);
  Inc(PC, 2);
end;

function TCpuBitWidth8.Read4: Cardinal;
begin
  Result := Memory.Read4(PC);
  Inc(PC, 4);
end;

function TCpuBitWidth8.Read8: DWord;
begin
  Result := Memory.Read8(PC);
  Inc(PC, 8);
end;

procedure TCpuBitWidth8.Reset;
begin
  PC := 0;
  A := 0;
end;

procedure TCpuBitWidth8.Run;
begin
  Reset;
  while not Halted do
    Step;
end;

procedure TCpuBitWidth8.Step;
var
  Opcode: TInstruction;
begin
  Opcode := TInstruction(Read1);
  case Opcode of
    inNop: ;
    inHalt: Halted := True;
    inLoad1: A := Read1;
    inLoad2: A := Read2;
    inLoad4: A := Read4;
    inLoad8: A := Read8;
    inLoadMem1: A := Memory.Read1(Read8);
    inLoadMem2: A := Memory.Read2(Read8);
    inLoadMem4: A := Memory.Read4(Read8);
    inLoadMem8: A := Memory.Read8(Read8);
    inStoreMem1: Memory.Write1(Read8, A);
    inStoreMem2: Memory.Write2(Read8, A);
    inStoreMem4: Memory.Write4(Read8, A);
    inStoreMem8: Memory.Write8(Read8, A);
    inInput1: A := IO.Read1(Read8);
    inInput2: A := IO.Read2(Read8);
    inInput4: A := IO.Read4(Read8);
    inInput8: A := IO.Read8(Read8);
    inOutput1: IO.Write1(Read8, A);
    inOutput2: IO.Write2(Read8, A);
    inOutput4: IO.Write4(Read8, A);
    inOutput8: IO.Write8(Read8, A);
  end;
end;

{ TCpuBitWidth1 }

function TCpuBitWidth1.Read1: Byte;
begin
  Result := Memory.Read1(PC);
  Inc(PC);
end;

procedure TCpuBitWidth1.Write1(Data: Byte);
begin
  Memory.Write1(PC, Data);
  Inc(PC);
end;

procedure TCpuBitWidth1.Reset;
begin
  PC := 0;
  A := 0;
end;

procedure TCpuBitWidth1.Run;
begin
  Reset;
  while not Halted do
    Step;
end;

procedure TCpuBitWidth1.Step;
var
  Opcode: TInstruction;
begin
  Opcode := TInstruction(Read1);
  case Opcode of
    inNop: ;
    inHalt: Halted := True;
    inLoad1: A := Read1;
    inLoadMem1: A := Memory.Read1(Read1);
    inStoreMem1: Memory.Write1(Read1, A);
    inInput1: A := IO.Read1(Read1);
    inOutput1: IO.Write1(Read1, A);
  end;
end;

end.

